@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2009 Red Hat Inc.
+ * Copyright (C) 2009-2012 Red Hat, Inc.
+ * Copyright (C) 2013 Nathan Hoad.
  *
- * Interface with iwlib from rhpl by Harald Hoyer <harald@redhat.com>.
+ * Interface with iwlib by Nathan Hoad <nathan@getoffmalawn.com>.
  *
  * This application is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -22,6 +23,8 @@
 #include <iwlib.h>
 #include <linux/sockios.h>
 
+#include "utils.h"
+
 /*------------------------------------------------------------------*/
 /*
  * Get wireless informations & config from the device driver
@@ -167,27 +170,18 @@ static PyObject *
 get_iwconfig (PyObject * self, PyObject * args)
 {
     char *devname;
-    int skfd;       /* generic raw socket desc. */
+    int skfd;
     char buffer[1024];
     int eno = 0;
-    struct wireless_info    info;
-    PyObject * dict;
+    struct wireless_info info;
 
     if (!PyArg_ParseTuple(args, "s", &devname)) {
         return NULL;
     }
 
-    /* Create a channel to the NET kernel. */
-    if((skfd = iw_sockets_open()) < 0) {
-        eno = errno;
-        sprintf(buffer, "iw_sockets_open [Errno %d] %s", eno, strerror(eno));
-        PyErr_SetString(PyExc_IOError, buffer);
-        return NULL;
-    }
-
+    OPEN_IW(skfd);
     eno = get_info(skfd, devname, &info);
-
-    close(skfd);
+    iw_sockets_close(skfd);
 
     if (eno < 0) {
         eno = -eno;
@@ -196,65 +190,87 @@ get_iwconfig (PyObject * self, PyObject * args)
         return NULL;
     }
 
-    dict = PyDict_New();
-    if (info.b.has_mode)
-        PyDict_SetItemString(dict, "Mode",
-                PyString_FromString(iw_operation_mode[info.b.mode]));
-    if (info.b.essid_on) {
-        PyDict_SetItemString(dict, "ESSID", PyString_FromString(info.b.essid));
-    } else {
-        PyDict_SetItemString(dict, "ESSID", PyString_FromString("Auto"));
+    return wireless_info_to_PyDict(&info);
+}
+
+static PyObject *
+set_essid(PyObject *self, PyObject *args)
+{
+    char *devname, *essid;
+    int sock;
+    int essid_len;
+    char buffer[1024];
+    struct iwreq wrq;
+    struct wireless_info info;
+    char essid_buf[IW_ESSID_MAX_SIZE+1];
+
+    if (!PyArg_ParseTuple(args, "ss", &devname, &essid)) {
+        return NULL;
     }
 
-    if (info.b.has_nwid) {
-        if(info.b.nwid.disabled)
-            PyDict_SetItemString(dict, "NWID", PyString_FromString("Auto"));
-        else
-            PyDict_SetItemString(dict, "NWID", PyString_FromFormat("%X", info.b.nwid.value));
-    }
+    essid_len = strlen(essid);
 
-    /* Display frequency / channel */
-    if(info.b.has_freq) {
-        iw_print_freq_value(buffer, sizeof(buffer), info.b.freq);
-        PyDict_SetItemString(dict, "Frequency", PyString_FromString(buffer));
-    }
+    OPEN_IW(sock);
 
-    /* Display the address of the current Access Point */
-    if(info.has_ap_addr) {
-        iw_saether_ntop((struct sockaddr *)info.ap_addr.sa_data, buffer);
-        /* Oups ! No Access Point in Ad-Hoc mode */
-        if((info.b.has_mode) && (info.b.mode == IW_MODE_ADHOC))
-            PyDict_SetItemString(dict, "Cell", PyString_FromString(buffer));
-        else
-            PyDict_SetItemString(dict, "Access Point", PyString_FromString(buffer));
-    }
-
-    /* Display the currently used/set bit-rate */
-    if(info.has_bitrate) {
-        /* Display it */
-        iw_print_bitrate(buffer, sizeof(buffer), info.bitrate.value);
-        PyDict_SetItemString(dict, "BitRate", PyString_FromString(buffer));
-        /*       printf("Bit Rate%c%s   ", (info.bitrate.fixed ? '=' : ':'), buffer); */
-    }
-
-    /* Display encryption information */
-    /* Note : we display only the "current" key, use iwlist to list all keys */
-    if(info.b.has_key) {
-        if((info.b.key_flags & IW_ENCODE_DISABLED) || (info.b.key_size == 0)) {
-            PyDict_SetItemString(dict, "Key", PyString_FromString("off"));
-        } else {
-            /* Display the key */
-            iw_print_key(buffer, sizeof(buffer), info.b.key, info.b.key_size, info.b.key_flags);
-            PyDict_SetItemString(dict, "Key", PyString_FromString(buffer));
+    if((!strcasecmp(essid, "off")) || (!strcasecmp(essid, "any"))) {
+        wrq.u.essid.flags = 0;
+        essid_buf[0] = '\0';
+    } else if (!strcasecmp(essid, "on")) {
+        memset(essid_buf, '\0', sizeof(essid_buf));
+        wrq.u.essid.pointer = (caddr_t) essid_buf;
+        wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
+        wrq.u.essid.flags = 0;
+        if(iw_get_ext(sock, devname, SIOCGIWESSID, &wrq) < 0) {
+            sprintf(buffer, "Error retrieving previous ESSID: %s",
+                strerror(errno));
+            goto error;
         }
+        wrq.u.essid.flags = 1;
+    } else if(essid_len > IW_ESSID_MAX_SIZE) {
+        sprintf(buffer, "ESSID is longer than the maximum %d",
+            IW_ESSID_MAX_SIZE);
+        goto error;
+    } else {
+        memcpy(essid_buf, essid, essid_len);
+        essid_buf[essid_len] = '\0';
+        wrq.u.essid.flags = 1;
     }
 
-    return(dict);
+    int we_kernel_version = iw_get_kernel_we_version();
+
+    wrq.u.essid.pointer = (caddr_t) essid_buf;
+    wrq.u.essid.length = strlen(essid_buf);
+    if(we_kernel_version < 21)
+        wrq.u.essid.length++;
+
+    if(iw_set_ext(sock, devname, SIOCSIWESSID, &wrq) < 0) {
+        sprintf(buffer, "Couldn't set essid on device '%s': %s", devname,
+            strerror(errno));
+        goto error;
+    }
+
+    iw_sockets_close(sock);
+    Py_INCREF(Py_None);
+    return Py_None;
+
+    error:
+    iw_sockets_close(sock);
+    PyErr_SetString(PyExc_OSError, buffer);
+    return NULL;
 }
 
 static struct PyMethodDef PyEthModuleMethods[] = {
     { "get_iwconfig",
-        (PyCFunction) get_iwconfig, METH_VARARGS, NULL },
+        (PyCFunction) get_iwconfig, METH_VARARGS,
+        "Retrieve the current configuration of an interface. \
+\n\nArguments:\
+\n  - device to work on (e.g. eth1, wlan0)." },
+    { "set_essid",
+        (PyCFunction) set_essid, METH_VARARGS,
+        "Set the ESSID of an interface. \
+\n\nArguments:\
+\n  - device to work on (e.g. eth1, wlan0).\
+\n  - essid to set device to." },
     { NULL, NULL, 0, NULL }
 };
 
